## How to Verify the Abella Development

The following instructions were tested on an Arch Linux System, though most steps should carry over to a MacOS System as well.

### Preparing the Execution Environment

First of all, you need the latest version of `opam` installed (from your OS package repository). At the time of writing this was `2.0.3`.

Make sure that you are working with an OCaml compiler at version `4.06.0`. Use `opam switch` to list available environments/OCaml compilers on your system. When a suitable switch exists, simply select it with, e.g.,

```
opam switch 4.06.0`
```
Otherwise you first need to create it with the following command.

```
opam switch create 4.06.0
```

Make sure that your opam environment is updated correctly. Simply follow the on-screen instructions after changing to a different switch. This usually requires executing `eval $(opam env)` in the active terminal.

### Installing Abella and Setting up Emacs Integration

We install a specific version of Abella into our opam switch
```
opam install abella.2.0.5
```

The Abella developers provide a fork of ProofGeneral (PG) for Emacs integration. Since the fork and the official version of PG are not really good at coexisting, it is adviseable to maintain a separate Emacs configuration on systems where PG is in regular use, e.g. due to ongoing Coq Development. We briefly describe a workable setup.

Firstly we fetch a copy of the Abella PG fork (verify that the `abella` branch was checked out):
```
git clone https://github.com/abella-prover/PG.git <ABELLA-PG>
cd <ABELLA-PG>

```

Next we create a copy of our `.emacs`,
```
cp ~/.emacs ~/.emacs-abella
```
and then add add the following lines to `.emacs-abella` (fix the two file paths, if necessary):
```
;; -------------- PROOF GENERAL - ABELLA FORK ------------------------
(defconst proof-site-file
  (expand-file-name "<ABELLA-PG>/generic/proof-site.el"))
(defconst lprolog-file
  (expand-file-name "~/.opam/4.06.0/share/abella/emacs/lprolog.el"))

(defmacro delete-mappings (alist key)
  `(while (assoc ,key ,alist)
     (setq ,alist (delq (assoc ,key ,alist) ,alist))))

(when (file-exists-p proof-site-file)
  (delete-mappings auto-mode-alist "\\.thm\\'")
  (setq proof-splash-enable nil)
  (setq proof-three-window-enable nil)
  (setq proof-three-window-mode-policy 'horizontal)
  (setq proof-output-tooltips nil)
  (load-file proof-site-file))

(when (file-exists-p lprolog-file)
  (autoload 'lprolog-mode lprolog-file "Major mode for Lambda Prolog" t)
  (delete-mappings auto-mode-alist "\\.mod\\'")
  (add-to-list 'auto-mode-alist '("\\.mod\\'" . lprolog-mode))
  (add-to-list 'auto-mode-alist '("\\.sig\\'" . lprolog-mode)))
```

### Check the Proof Script

To verify the proof from the terminal, simply run the following.
```
abella equiv.thm
```

To be able to step through the proof, launch emacs with the PG fork and the custom config that we prepared above.
```
emacs -q -l /home/jonas/.emacs-abella equiv.thm
```

In case this is done repeatedly, it helps to define a bash alias. Add this to your `.bash_aliases`file (or alternatively to `.bashrc`, `.bash_profile` or other script which is sourced on terminal setup).
```
alias emacs-abella="emacs -q -l /home/jonas/.emacs-abella"
```
Then simply run
```
emacs-abella equiv.thm
```
The PG key bindings are as expected.
